@@ -2,11 +2,14 @@ import dataclasses
 
 import asyncio
 import base64
-import click
 import datetime
 import json
 import sys
-from neuromation.api import Client, Container, HTTPPort, JobStatus, Resources
+import uuid
+from neuromation.api import Client, Container, HTTPPort, Resources, Volume
+from neuromation.cli.job import STORAGE_MOUNTPOINT
+from neuromation.cli.utils import NEURO_STEAL_CONFIG
+from rich.console import Console
 from typing import (
     AbstractSet,
     Dict,
@@ -18,6 +21,7 @@ from typing import (
     Tuple,
     TypeVar,
 )
+from yarl import URL
 
 from .commands import CmdProcessor
 from .config_loader import BatchRemoteCL
@@ -35,7 +39,7 @@ from .context import (
 )
 from .storage import Attempt, BatchStorage, FinishedTask, StartedTask
 from .types import AlwaysT, FullID, TaskStatus
-from .utils import TERMINATED_JOB_STATUSES, fmt_id, fmt_raw_id, fmt_status
+from .utils import TERMINATED_JOB_STATUSES, TERMINATED_TASK_STATUSES, fmt_id, fmt_raw_id
 
 
 if sys.version_info >= (3, 9):
@@ -216,6 +220,7 @@ class BakeTasksManager:
 class BatchExecutor:
     def __init__(
         self,
+        console: Console,
         flow: RunningBatchFlow,
         attempt: Attempt,
         client: Client,
@@ -223,6 +228,7 @@ class BatchExecutor:
         *,
         polling_timeout: float = 1,
     ) -> None:
+        self._console = console
         self._top_flow = flow
         self._attempt = attempt
         self._client = client
@@ -243,13 +249,14 @@ class BatchExecutor:
     @classmethod
     async def create(
         cls,
+        console: Console,
         executor_data: ExecutorData,
         client: Client,
         storage: BatchStorage,
         *,
         polling_timeout: float = 1,
     ) -> "BatchExecutor":
-        click.echo("Fetch bake data")
+        console.print("Fetch bake data")
         bake = await storage.fetch_bake(
             executor_data.project,
             executor_data.batch,
@@ -257,7 +264,7 @@ class BatchExecutor:
             executor_data.suffix,
         )
 
-        click.echo("Fetch configs metadata")
+        console.print("Fetch configs metadata")
         meta = await storage.fetch_configs_meta(bake)
         config_loader = BatchRemoteCL(
             meta,
@@ -265,10 +272,11 @@ class BatchExecutor:
         )
         flow = await RunningBatchFlow.create(config_loader, bake.batch, bake.params)
 
-        click.echo("Find last attempt")
+        console.print("Find last attempt")
         attempt = await storage.find_attempt(bake)
 
         return cls(
+            console,
             flow,
             attempt,
             client,
@@ -322,8 +330,7 @@ class BatchExecutor:
         st, ft = await self._storage.skip_task(self._attempt, full_id)
         self._tasks_mgr.add_started(st)
         self._mark_finished(ft)
-        str_skipped = click.style("skipped", fg="magenta")
-        click.echo(f"Task {fmt_id(full_id)} is {str_skipped}")
+        self._console.print("Task", fmt_id(full_id), "is", TaskStatus.SKIPPED)
 
     async def _process_local(self, full_id: FullID) -> None:
         raise ValueError("Processing of local actions is not supported")
@@ -332,8 +339,7 @@ class BatchExecutor:
         st = await self._storage.start_action(self._attempt, full_id)
         self._tasks_mgr.add_started(st)
         await self._embed_action(full_id)
-        str_started = click.style("started", fg="cyan")
-        click.echo(f"Action {fmt_id(st.id)} is {str_started}")
+        self._console.print(f"Action", fmt_id(st.id), "is", TaskStatus.PENDING)
 
     async def _process_task(self, full_id: FullID) -> None:
 
@@ -359,18 +365,17 @@ class BatchExecutor:
             )
 
         if ft is not None:
-            str_cached = click.style("cached", fg="magenta")
-            click.echo(
-                f"Task {fmt_id(ft.id)} [{fmt_raw_id(ft.raw_id)}] " f"is {str_cached}"
+            self._console.print(
+                "Task", fmt_id(ft.id), fmt_raw_id(ft.raw_id), "is", TaskStatus.CACHED
             )
             assert ft.status == TaskStatus.SUCCEEDED
             self._mark_finished(ft)
         else:
             st = await self._start_task(full_id, task)
             self._tasks_mgr.add_started(st)
-            str_started = click.style("started", fg="cyan")
-            raw_id = fmt_raw_id(st.raw_id)
-            click.echo(f"Task {fmt_id(st.id)} [{raw_id}] is {str_started}")
+            self._console.print(
+                "Task", fmt_id(st.id), fmt_raw_id(st.raw_id), "is", TaskStatus.PENDING
+            )
 
     async def _load_previous_run(self) -> None:
         # Loads tasks that previous executor run processed.
@@ -427,7 +432,7 @@ class BatchExecutor:
             if not self._is_cancelling:
                 await self._refresh_attempt()
 
-                if not ok or self._attempt.result == JobStatus.CANCELLED:
+                if not ok or self._attempt.result == TaskStatus.CANCELLED:
                     self._is_cancelling = True
                     await self._stop_running()
 
@@ -437,16 +442,15 @@ class BatchExecutor:
 
     async def _finish_run(self) -> None:
         attempt_status = self._accumulate_result()
-        str_attempt_status = fmt_status(attempt_status)
-        if self._attempt.result not in TERMINATED_JOB_STATUSES:
+        if self._attempt.result not in TERMINATED_TASK_STATUSES:
             await self._storage.finish_attempt(self._attempt, attempt_status)
-        click.echo(f"Attempt #{self._attempt.number} {str_attempt_status}")
+        self._console.print(f"[b]Attempt #{self._attempt.number}[/b]", attempt_status)
 
     async def _stop_running(self) -> None:
         for full_id, st in self._tasks_mgr.running_tasks.items():
             task = await self._get_task(full_id)
             if task.enable is not AlwaysT():
-                click.echo(f"Task {fmt_id(st.id)} is being killed")
+                self._console.print(f"Task {fmt_id(st.id)} is being killed")
                 await self._client.jobs.kill(st.raw_id)
 
     async def _store_to_cache(self, ft: FinishedTask) -> None:
@@ -477,14 +481,16 @@ class BatchExecutor:
 
                 await self._store_to_cache(ft)
 
-                str_status = fmt_status(ft.status)
-                raw_id = fmt_raw_id(ft.raw_id)
-                click.echo(
-                    f"Task {fmt_id(ft.id)} [{raw_id}] is {str_status}"
-                    + (" with following outputs:" if ft.outputs else "")
+                self._console.print(
+                    "Task",
+                    fmt_id(ft.id),
+                    fmt_raw_id(ft.raw_id),
+                    "is",
+                    ft.status,
+                    (" with following outputs:" if ft.outputs else ""),
                 )
                 for key, value in ft.outputs.items():
-                    click.echo(f"  {key}: {value}")
+                    self._console.print(f"  {key}: {value}")
 
                 task_meta = await self._get_meta(full_id)
                 if ft.status != TaskStatus.SUCCEEDED and task_meta.strategy.fail_fast:
@@ -504,27 +510,27 @@ class BatchExecutor:
             )
             self._mark_finished(ft)
 
-            str_status = fmt_status(ft.status)
-            click.echo(
-                f"Action {fmt_id(ft.id)} is {str_status}"
-                + (" with following outputs:" if ft.outputs else "")
+            self._console.print(
+                f"Action {fmt_id(ft.id)} is",
+                ft.status,
+                (" with following outputs:" if ft.outputs else ""),
             )
             for key, value in ft.outputs.items():
-                click.echo(f"  {key}: {value}")
+                self._console.print(f"  {key}: {value}")
 
             task_meta = await self._get_meta(full_id)
             if ft.status != TaskStatus.SUCCEEDED and task_meta.strategy.fail_fast:
                 return False
         return True
 
-    def _accumulate_result(self) -> JobStatus:
+    def _accumulate_result(self) -> TaskStatus:
         for task in self._tasks_mgr.finished.values():
             if task.status == TaskStatus.CANCELLED:
-                return JobStatus.CANCELLED
+                return TaskStatus.CANCELLED
             elif task.status == TaskStatus.FAILED:
-                return JobStatus.FAILED
+                return TaskStatus.FAILED
 
-        return JobStatus.SUCCEEDED
+        return TaskStatus.SUCCEEDED
 
     async def _start_task(self, full_id: FullID, task: Task) -> StartedTask:
 
@@ -546,11 +552,22 @@ class BatchExecutor:
             tpu_software_version=preset.tpu_software_version,
         )
         volumes_parsed = self._client.parse.volumes(task.volumes)
+        volumes = list(volumes_parsed.volumes)
 
         http_auth = task.http_auth
         if http_auth is None:
             http_auth = HTTPPort.requires_auth
 
+        if task.pass_config:
+            env_name = NEURO_STEAL_CONFIG
+            if env_name in env_dict:
+                raise ValueError(f"{env_name} is already set to {env_dict[env_name]}")
+            env_var, secret_volume = await upload_and_map_config(
+                self._console, self._client
+            )
+            env_dict[NEURO_STEAL_CONFIG] = env_var
+            volumes.append(secret_volume)
+
         container = Container(
             image=self._client.parse.remote_image(task.image),
             entrypoint=task.entrypoint,
@@ -559,7 +576,7 @@ class BatchExecutor:
             resources=resources,
             env=env_dict,
             secret_env=secret_env_dict,
-            volumes=list(volumes_parsed.volumes),
+            volumes=volumes,
             secret_files=list(volumes_parsed.secret_files),
             disk_volumes=list(volumes_parsed.disk_volumes),
             tty=False,
@@ -579,6 +596,7 @@ class LocalsBatchExecutor(BatchExecutor):
     @classmethod
     async def create(
         cls,
+        console: Console,
         executor_data: ExecutorData,
         client: Client,
         storage: BatchStorage,
@@ -589,15 +607,14 @@ class LocalsBatchExecutor(BatchExecutor):
             polling_timeout is None
         ), "polling_timeout is disabled for LocalsBatchExecutor"
         return await super(cls, LocalsBatchExecutor).create(
-            executor_data, client, storage, polling_timeout=0
+            console, executor_data, client, storage, polling_timeout=0
         )
 
     async def _process_local(self, full_id: FullID) -> None:
         local = await self._get_local(full_id)
         st = await self._storage.start_action(self._attempt, full_id)
         self._tasks_mgr.add_started(st)
-        str_started = click.style("started", fg="cyan")
-        click.echo(f"Local action {fmt_id(st.id)} is {str_started}")
+        self._console.print(f"Local action {fmt_id(st.id)} is", TaskStatus.PENDING)
 
         subprocess = await asyncio.create_subprocess_shell(
             local.cmd,
@@ -607,9 +624,9 @@ class LocalsBatchExecutor(BatchExecutor):
         (stdout_data, stderr_data) = await subprocess.communicate()
         async with CmdProcessor() as proc:
             async for line in proc.feed_chunk(stdout_data):
-                click.echo(line)
+                self._console.print(line)
             async for line in proc.feed_eof():
-                click.echo(line)
+                self._console.print(line)
         if subprocess.returncode == 0:
             result_status = TaskStatus.SUCCEEDED
         else:
@@ -624,13 +641,13 @@ class LocalsBatchExecutor(BatchExecutor):
         )
         self._mark_finished(ft)
 
-        str_status = fmt_status(ft.status)
-        click.echo(
-            f"Action {fmt_id(ft.id)} is {str_status}"
-            + (" with following outputs:" if ft.outputs else "")
+        self._console.print(
+            f"Action {fmt_id(ft.id)} is",
+            ft.status,
+            (" with following outputs:" if ft.outputs else ""),
         )
         for key, value in ft.outputs.items():
-            click.echo(f"  {key}: {value}")
+            self._console.print(f"  {key}: {value}")
 
     async def _process_action(self, full_id: FullID) -> None:
         pass  # Skip for local
@@ -646,3 +663,38 @@ class LocalsBatchExecutor(BatchExecutor):
 
     async def _finish_run(self) -> None:
         pass  # Do nothing for locals run
+
+
+async def upload_and_map_config(console: Console, client: Client) -> Tuple[str, Volume]:
+
+    # store the Neuro CLI config on the storage under some random path
+    nmrc_path = URL(client.config._path.expanduser().resolve().as_uri())
+    random_nmrc_filename = f"{uuid.uuid4()}-cfg"
+    storage_nmrc_folder = URL(
+        f"storage://{client.config.cluster_name}/{client.config.username}/.neuro/"
+    )
+    storage_nmrc_path = storage_nmrc_folder / random_nmrc_filename
+    local_nmrc_folder = f"{STORAGE_MOUNTPOINT}/.neuro/"
+    local_nmrc_path = f"{local_nmrc_folder}{random_nmrc_filename}"
+    console.print(
+        f"[bright_black]Temporary config file created on storage: {storage_nmrc_path}."
+    )
+    console.print(
+        f"[bright_black]Inside container it will be available at: {local_nmrc_path}."
+    )
+    await client.storage.mkdir(storage_nmrc_folder, parents=True, exist_ok=True)
+
+    async def skip_tmp(fname: str) -> bool:
+        return not fname.endswith(("-shm", "-wal", "-journal"))
+
+    await client.storage.upload_dir(nmrc_path, storage_nmrc_path, filter=skip_tmp)
+    # specify a container volume and mount the storage path
+    # into specific container path
+    return (
+        local_nmrc_path,
+        Volume(
+            storage_uri=storage_nmrc_folder,
+            container_path=local_nmrc_folder,
+            read_only=False,
+        ),
+    )
