@@ -8,7 +8,9 @@ import secrets
 import shlex
 import sys
 from neuromation.api import Client, Factory, JobDescription, JobStatus, ResourceNotFound
-from neuromation.cli.formatters import ftable  # TODO: extract into a separate library
+from rich import box
+from rich.console import Console
+from rich.table import Table
 from types import TracebackType
 from typing import (
     AbstractSet,
@@ -25,14 +27,8 @@ from typing_extensions import AsyncContextManager
 from .config_loader import LiveLocalCL
 from .context import ImageCtx, JobMeta, RunningLiveFlow, UnknownJob, VolumeCtx
 from .parser import ConfigDir
-from .utils import (
-    RUNNING_JOB_STATUSES,
-    TERMINATED_JOB_STATUSES,
-    fmt_id,
-    fmt_raw_id,
-    fmt_status,
-    run_subproc,
-)
+from .types import TaskStatus
+from .utils import RUNNING_JOB_STATUSES, TERMINATED_JOB_STATUSES, fmt_id, run_subproc
 
 
 @dataclasses.dataclass(frozen=True)
@@ -45,8 +41,9 @@ class JobInfo:
 
 
 class LiveRunner(AsyncContextManager["LiveRunner"]):
-    def __init__(self, config_dir: ConfigDir) -> None:
+    def __init__(self, config_dir: ConfigDir, console: Console) -> None:
         self._config_dir = config_dir
+        self._console = console
         self._config_loader: Optional[LiveLocalCL] = None
         self._flow: Optional[RunningLiveFlow] = None
         self._client: Optional[Client] = None
@@ -104,9 +101,9 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
                     )
             return meta
         except UnknownJob:
-            click.secho(f"Unknown job {fmt_id(job_id)}", fg="red")
+            self._console.print(f"[red]Unknown job [b]{job_id}[/b]")
             jobs_str = ",".join(self.flow.job_ids)
-            click.secho(f"Existing jobs: {jobs_str}", dim=True)
+            self._console.print(f"[dim]Existing jobs: {jobs_str}")
             sys.exit(1)
 
     async def _resolve_jobs(
@@ -185,15 +182,12 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
 
         # TODO: make concurent queries for job statuses
         loop = asyncio.get_event_loop()
-        rows: List[List[str]] = []
-        rows.append(
-            [
-                click.style("JOB", bold=True),
-                click.style("STATUS", bold=True),
-                click.style("RAW ID", bold=True),
-                click.style("WHEN", bold=True),
-            ]
-        )
+        table = Table(box=box.MINIMAL_HEAVY_HEAD)
+        table.add_column("JOB", style="bold")
+        table.add_column("STATUS")
+        table.add_column("RAW ID", style="bright_black")
+        table.add_column("WHEN")
+
         tasks = []
         for job_id in self.flow.job_ids:
             tasks.append(loop.create_task(self._job_status(job_id)))
@@ -208,17 +202,14 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
                         when_humanized = humanize.naturaltime(delta)
                     else:
                         when_humanized = humanize.naturaldate(info.when.astimezone())
-                rows.append(
-                    [
-                        fmt_id(info.id),
-                        fmt_status(info.status),
-                        fmt_raw_id(info.raw_id or "N/A"),
-                        when_humanized,
-                    ]
+                table.add_row(
+                    info.id,
+                    TaskStatus(info.status),
+                    info.raw_id or "N/A",
+                    when_humanized,
                 )
 
-        for line in ftable.table(rows):
-            click.echo(line)
+        self._console.print(table)
 
     async def status(self, job_id: str, suffix: Optional[str]) -> None:
         meta_ctx = await self._ensure_meta(job_id, suffix)
@@ -226,7 +217,7 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
             async for descr in self._resolve_jobs(meta_ctx, suffix):
                 await run_subproc("neuro", "status", descr.id)
         except ResourceNotFound:
-            click.echo(f"Job {fmt_id(job_id)} is not running")
+            self._console.print(f"Job [b]{job_id}[/b] is not running")
             sys.exit(1)
 
     async def _try_attach_to_running(
@@ -258,10 +249,14 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
                         "Multi job with such suffix is already running."
                     )
                 if descr.status == JobStatus.PENDING:
-                    click.echo(f"Job {fmt_id(job_id)} is pending, " "try again later")
+                    self._console.print(
+                        f"Job [b]{job_id}[/b] is pending, " "try again later"
+                    )
                     sys.exit(2)
                 if descr.status == JobStatus.RUNNING:
-                    click.echo(f"Job {fmt_id(job_id)} is running, " "connecting...")
+                    self._console.print(
+                        f"Job [b]{job_id}[/b] is running, " "connecting..."
+                    )
                     if not is_multi:
                         job = await self.flow.get_job(job_id, params)
                     else:
@@ -337,6 +332,8 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
             run_args.append(f"--detach")
         for pf in job.port_forward:
             run_args.append(f"--port-forward={pf}")
+        if job.pass_config:
+            run_args.append(f"--pass-config")
 
         run_args.append(job.image)
         if job.cmd:
@@ -354,7 +351,7 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
             async for descr in self._resolve_jobs(meta_ctx, suffix):
                 await run_subproc("neuro", "logs", descr.id)
         except ResourceNotFound:
-            click.echo(f"Job {fmt_id(job_id)} is not running")
+            self._console.print(f"Job {fmt_id(job_id)} is not running")
             sys.exit(1)
 
     async def kill_job(self, job_id: str, suffix: Optional[str]) -> bool:
@@ -377,9 +374,9 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
     async def kill(self, job_id: str, suffix: Optional[str]) -> None:
         """Kill named job"""
         if await self.kill_job(job_id, suffix):
-            click.echo(f"Killed job {fmt_id(job_id)}")
+            self._console.print(f"Killed job [b]{job_id}[/b]")
         else:
-            click.echo(f"Job {fmt_id(job_id)} is not running")
+            self._console.print(f"Job [b]{job_id}[/b] is not running")
 
     async def kill_all(self) -> None:
         """Kill all jobs"""
@@ -412,21 +409,21 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
         ):
             tasks.append(loop.create_task(kill(descr)))
 
-        for job_id, ret in await asyncio.gather(*tasks):
-            click.echo(f"Killed job {fmt_id(job_id)}")
+        for job_info in await asyncio.gather(*tasks):
+            self._console.print(f"Killed job [b]{job_info}[/b]")
 
     # volumes subsystem
 
     async def find_volume(self, volume: str) -> VolumeCtx:
         volume_ctx = self.flow.volumes.get(volume)
         if volume_ctx is None:
-            click.secho(f"Unknown volume {click.style(volume, bold=True)}", fg="red")
+            self._console.print(f"[red]Unknown volume [b]{volume}[/b]")
             volumes = sorted([volume for volume in self.flow.volumes.keys()])
             volumes_str = ",".join(volumes)
-            click.secho(f"Existing volumes: {volumes_str}", dim=True)
+            self._console.print(f"[dim]Existing volumes: {volumes_str}")
             sys.exit(1)
         if volume_ctx.local is None:
-            click.echo(f"Volume's {click.style('local', bold=True)} part is not set")
+            self._console.print(f"Volume's [b]local[/b] part is not set")
             sys.exit(2)
         return volume_ctx
 
@@ -477,7 +474,7 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
         for volume in self.flow.volumes.values():
             if volume.local is not None:
                 volume_ctx = await self.find_volume(volume.id)
-                click.echo(f"Create volume {fmt_id(volume.id)}")
+                self._console.print(f"Create volume [b]{volume.id}[/b]")
                 await run_subproc(
                     "neuro",
                     "mkdir",
@@ -490,18 +487,16 @@ class LiveRunner(AsyncContextManager["LiveRunner"]):
     async def find_image(self, image: str) -> ImageCtx:
         image_ctx = self.flow.images.get(image)
         if image_ctx is None:
-            click.secho(f"Unknown image {click.style(image, bold=True)}", fg="red")
+            self._console.print(f"[red]Unknown image [b]{image}[/b]")
             images = sorted([image for image in self.flow.images.keys()])
             images_str = ",".join(images)
-            click.secho(f"Existing images: {images_str}", dim=True)
+            self._console.print(f"[dim]Existing images: {images_str}")
             sys.exit(1)
         if image_ctx.context is None:
-            click.echo(f"Image's {click.style('context', bold=True)} part is not set")
+            self._console.print(f"Image's [b]context[/b] part is not set")
             sys.exit(2)
         if image_ctx.dockerfile is None:
-            click.echo(
-                f"Image's {click.style('dockerfile', bold=True)} part is not set"
-            )
+            self._console.print(f"Image's [b]dockerfile[/b] part is not set")
             sys.exit(3)
         return image_ctx
 
