@@ -1,20 +1,8 @@
 import asyncio
-import click
 from neuromation.api import JobStatus
 from typing import Union
 
-from .types import FullID, TaskStatus
-
-
-COLORS = {
-    TaskStatus.PENDING: "cyan",
-    TaskStatus.RUNNING: "blue",
-    TaskStatus.SUCCEEDED: "green",
-    TaskStatus.CANCELLED: "yellow",
-    TaskStatus.SKIPPED: "magenta",
-    TaskStatus.FAILED: "red",
-    TaskStatus.UNKNOWN: "bright_black",
-}
+from .types import COLORS, FullID, TaskStatus
 
 
 def fmt_status(status: Union[JobStatus, TaskStatus]) -> str:
@@ -22,7 +10,8 @@ def fmt_status(status: Union[JobStatus, TaskStatus]) -> str:
         status = TaskStatus(status)
     else:
         assert isinstance(status, TaskStatus)
-    return click.style(status.value, fg=COLORS.get(status, "reset"))
+    color = COLORS.get(status, "none")
+    return f"[{color}]{status.value}[/{color}]"
 
 
 def fmt_id(id: Union[str, FullID]) -> str:
@@ -30,14 +19,15 @@ def fmt_id(id: Union[str, FullID]) -> str:
         s_id = id
     else:
         s_id = ".".join(id)
-    return click.style(s_id, bold=True)
+    return f"[b]{s_id}[/b]"
 
 
 def fmt_raw_id(raw_id: str) -> str:
-    return click.style(raw_id, dim=True)
+    return f"[bright_black]{raw_id}[/bright_black]"
 
 
 RUNNING_JOB_STATUSES = {JobStatus.PENDING, JobStatus.RUNNING}
+RUNNING_TASK_STATUSES = {TaskStatus.PENDING, TaskStatus.RUNNING}
 
 TERMINATED_JOB_STATUSES = {
     JobStatus.FAILED,
@@ -45,6 +35,12 @@ TERMINATED_JOB_STATUSES = {
     JobStatus.CANCELLED,
 }
 
+TERMINATED_TASK_STATUSES = {
+    TaskStatus.FAILED,
+    TaskStatus.SUCCEEDED,
+    TaskStatus.CANCELLED,
+}
+
 
 JOB_TAG_PATTERN = r"\A[a-z](?:[-.:/]?[a-z0-9]){0,255}\Z"
 
